@@ -41,6 +41,8 @@ empathy_contact_list_get_type (void)
 		type = g_type_register_static (G_TYPE_INTERFACE,
 					       "EmpathyContactList",
 					       &type_info, 0);
+
+		g_type_interface_add_prerequisite (type, G_TYPE_OBJECT);
 	}
 
 	return type;
@@ -57,7 +59,7 @@ contact_list_base_init (gpointer klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      empathy_marshal_VOID__OBJECT_OBJECT_UINT_STRING_BOOLEAN,
+			      _empathy_marshal_VOID__OBJECT_OBJECT_UINT_STRING_BOOLEAN,
 			      G_TYPE_NONE,
 			      5, EMPATHY_TYPE_CONTACT, EMPATHY_TYPE_CONTACT,
 			      G_TYPE_UINT, G_TYPE_STRING, G_TYPE_BOOLEAN);
@@ -67,7 +69,7 @@ contact_list_base_init (gpointer klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      empathy_marshal_VOID__OBJECT_OBJECT_UINT_STRING_BOOLEAN,
+			      _empathy_marshal_VOID__OBJECT_OBJECT_UINT_STRING_BOOLEAN,
 			      G_TYPE_NONE,
 			      5, EMPATHY_TYPE_CONTACT, EMPATHY_TYPE_CONTACT,
 			      G_TYPE_UINT, G_TYPE_STRING, G_TYPE_BOOLEAN);
@@ -77,7 +79,7 @@ contact_list_base_init (gpointer klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      empathy_marshal_VOID__OBJECT_STRING_BOOLEAN,
+			      _empathy_marshal_VOID__OBJECT_STRING_BOOLEAN,
 			      G_TYPE_NONE,
 			      3, EMPATHY_TYPE_CONTACT, G_TYPE_STRING, G_TYPE_BOOLEAN);
 
